# Example 2 in the manuscript: vowel data 

options(width=68, digits=5)

# Load VGAM for R 
library(VGAM, lib.loc = "/home/yee/myRlibs")  # Change path!

# Read in the vowel data

source("./vowel.char.R")
source("./vowel.train.R")

x = vowel.train$x        # 528x10 matrix
y = vowel.train$y        # vector with values in 1:11

x = scale(x)   # Center and scale x 

# Full MLM model
# Note: the matrices involved are large!
# Also, these data are computationally expensive

full = vglm(y ~ x, multinomial, trace=T, save.wei=T)
my.coef =  coef(full, matrix=T)
print(t(coef(full, matrix=T)), dig=3)
vc = summary(full)@cov.unscaled
my.se =  matrix(sqrt(diag(vc)), nrow(my.coef), byrow=T)
print(t(my.coef/my.se), dig=3)     # t-statistics

sfull = summary(full, disper=0)       # Estimate the scaling parameter
scaling.parameter = sfull@dispersion  # NOT assumed to be unity

# -------------------------------------------------------------------

# Reduced-rank models (RR-MLMs)

u0 = vglm(y ~ 1, multinomial, rank=1, Svd=T, Uncor=T, trace=T)
print(t(coef(u0, matrix=T)), dig=3)

set.seed(123)
u1 = rrvglm(y ~ x, multinomial, rank=1, Svd=T, Uncor=T, trace=T)
u1@constraints[[2]]
print(t(coef(u1, matrix=T)), dig=3)

set.seed(123)
u2 = rrvglm(y ~ x, multinomial, rank=2, Svd=T, Uncor=T, trace=T)
u2@constraints[[2]]
print(t(coef(u2, matrix=T)), dig=3)

set.seed(213)
u3 = rrvglm(y ~ x, multinomial, rank=3, Svd=T, Uncor=T, trace=T)
u3@constraints[[2]]
print(t(coef(u3, matrix=T)), dig=3)

set.seed(123)
u4 = rrvglm(y ~ x, multinomial, rank=4, Svd=T, Uncor=T, trace=T)
u4@constraints[[2]]
print(t(coef(u4, matrix=T)), dig=3)

set.seed(123)
u5 = rrvglm(y ~ x, multinomial, rank=5, Svd=T, Uncor=T, trace=T)
u5@constraints[[2]]
print(t(coef(u5, matrix=T)), dig=3)

set.seed(123)
u6 = rrvglm(y ~ x, multinomial, rank=6, Svd=T, Uncor=T, trace=T)
u6@constraints[[2]]
print(t(coef(u6, matrix=T)), dig=3)

set.seed(123)
u7 = rrvglm(y ~ x, multinomial, rank=7, Svd=T, Uncor=T, trace=T)
u7@constraints[[2]]
print(t(coef(u7, matrix=T)), dig=3)

set.seed(123)
u8 = rrvglm(y ~ x, multinomial, rank=8, Svd=T, Uncor=T, trace=T)
u8@constraints[[2]]
print(t(coef(u8, matrix=T)), dig=3)

set.seed(123)
u9 = rrvglm(y ~ x, multinomial, rank=9, Svd=T, Uncor=T, trace=T)
u9@constraints[[2]]
print(t(coef(u9, matrix=T)), dig=3)


# -------------------------------------------------------------------

# Full model vs null model (exact test)
1-pchisq((u0@criterion$deviance - full@criterion$deviance)/scaling.parameter,
         df=length(coef(full)) - length(coef(u0)))


# -------------------------------------------------------------------
# Do naive likelihood ratio test based on unnested models

# Deviances collected manually 
Du = c(2532.177, 1677.292, 1052.039, 921.8816, 828.1246, 729.8645,
       697.5122, 685.659, 680.509, 677.037, 676.998)  # short-cut
np = c(10,29,46,61,74,85,94,101,106,109,110) # 'Number' of parameters in B_1,A&C

log.liks = c(u0@criterion$loglikelihood,
             u1@criterion$loglikelihood,
             u2@criterion$loglikelihood,
             u3@criterion$loglikelihood,
             u4@criterion$loglikelihood,
             u5@criterion$loglikelihood,
             u6@criterion$loglikelihood,
             u7@criterion$loglikelihood,
             u8@criterion$loglikelihood,
             u9@criterion$loglikelihood,
             full@criterion$loglikelihood)
p2 = 10  # Number of covariates involved in the reduced-rank regression
p1 = 1   # Just the intercept which is not involved 
M = u2@misc$M
r = 0:10
np.check = M + r*p2 + (M-r)*r
np.check - np    # Should be all 0's

# -------------------------------------------------------------------
# Identification of the rank (based on AIC)

my.aic = -2*log.liks + 2 * np * scaling.parameter
cbind(r, my.aic)
plot(r, my.aic, type="n", ylab="AIC", xlab="Rank r")
points(r, my.aic, type="b", lwd=2)
plot(np, my.aic, type="n", ylab="AIC", xlab="Number of `parameters'")
points(np, my.aic, type="b", lwd=2)


r = 0:10
# Print out P-values (yardstick only)
for(r in 2:length(Du)) {
    print(c(r=r-1, pvalue=1-pchisq((Du[r-1]-Du[r])/scaling.parameter,
                                   df=np[r]-np[r-1])))
    cat("\n")
}


# -------------------------------------------------------------------


postscript('./vowelplot2.eps',
           horiz=F, width=6.6, height=6.5,
           onefile=FALSE, print.it=FALSE, pointsize=12)
par(las=1, mar=c(5,4,1,0.2)+0.1)
lvplot(u2, y, pch=vowel.char[y], chull=T, clty=1:11)

if(F) {
    # Fancier plot for computer screens 
    lvplot(u2, y, chull=T,
           pch=vowel.char[y], pcol=(1:13)[y],
           clty=1:13, ccol=1:13) 
}

dev.off() 


postscript('./vowelbiplot1.eps',
           horiz=F, width=6.6, height=6.5,
           onefile=FALSE, print.it=FALSE, pointsize=12)
par(las=1, mar=c(3,3,1,0.2)+0.1)
# Use vowels because putting a "3" at the origin is a little messy 
vowels = vowel.char
vowels[length(vowels)] = " "  # Obscure the origin temporarily 
biplot(u2, delta=0.2, scale=0.2, legend=vowels,
       xlab="", ylab="", xlim=c(-3.1, 2.3), ylim=c(-2.5, 3.4),
       alabels=paste("x", 2:11)) 
# Nb. x1,...,x10 changed to x2,...,x11 to keep accordance to the 
# notation in the paper
text(0.15, 0.05, vowel.char[length(vowel.char)])  # Label the origin
dev.off() 




q()

